@@ -533,7 +533,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
 
     @Override
     public Collection<Part> getParts() throws IOException, ServletException {
-        verifyMultipartServlet();
         if (parts == null) {
             loadParts();
         }
@@ -550,11 +549,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
 
     @Override
     public Part getPart(final String name) throws IOException, ServletException {
-        verifyMultipartServlet();
-        if (parts == null) {
-            loadParts();
-        }
-        for (Part part : parts) {
+        for (Part part : getParts()) {
             if (part.getName().equals(name)) {
                 return part;
             }
@@ -580,6 +575,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         final ServletRequestContext requestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
 
         if (parts == null) {
+            verifyMultipartServlet();
             final List<Part> parts = new ArrayList<>();
             String mimeType = exchange.getRequestHeaders().getFirst(Headers.CONTENT_TYPE);
             if (mimeType != null && mimeType.startsWith(MultiPartParserDefinition.MULTIPART_FORM_DATA)) {
@@ -738,7 +734,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             final FormData parsedFormData = parseFormData();
             if (parsedFormData != null) {
                 FormData.FormValue res = parsedFormData.getFirst(name);
-                if (res == null || res.isFileItem()) {
+                if (res == null || res.isFileItem() && !res.isBigField()) {
                     return null;
                 } else {
                     return res.getValue();
@@ -761,7 +757,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             while (it.hasNext()) {
                 String name = it.next();
                 for(FormData.FormValue param : parsedFormData.get(name)) {
-                    if(!param.isFileItem()) {
+                    if(!param.isFileItem() || param.isBigField()) {
                         parameterNames.add(name);
                         break;
                     }
@@ -788,7 +784,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
             Deque<FormData.FormValue> res = parsedFormData.get(name);
             if (res != null) {
                 for (FormData.FormValue value : res) {
-                    if(!value.isFileItem()) {
+                    if(!value.isFileItem() || value.isBigField()) {
                         ret.add(value.getValue());
                     }
                 }
@@ -819,14 +815,14 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
                 if (arrayMap.containsKey(name)) {
                     ArrayList<String> existing = arrayMap.get(name);
                     for (final FormData.FormValue v : val) {
-                        if(!v.isFileItem()) {
+                        if(!v.isFileItem() || v.isBigField()) {
                             existing.add(v.getValue());
                         }
                     }
                 } else {
                     final ArrayList<String> values = new ArrayList<>();
                     for (final FormData.FormValue v : val) {
-                        if(!v.isFileItem()) {
+                        if(!v.isFileItem() || v.isBigField()) {
                             values.add(v.getValue());
                         }
                     }
@@ -1060,9 +1056,10 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         } else if (asyncStarted) {
             throw UndertowServletMessages.MESSAGES.asyncAlreadyStarted();
         }
-        asyncStarted = true;
         final ServletRequestContext servletRequestContext = exchange.getAttachment(ServletRequestContext.ATTACHMENT_KEY);
-        return asyncContext = new AsyncContextImpl(exchange, servletRequestContext.getServletRequest(), servletRequestContext.getServletResponse(), servletRequestContext, false, asyncContext);
+        asyncContext = new AsyncContextImpl(exchange, servletRequestContext.getServletRequest(), servletRequestContext.getServletResponse(), servletRequestContext, false, asyncContext);
+        asyncStarted = true;
+        return asyncContext;
     }
 
     @Override
@@ -1085,10 +1082,11 @@ public final class HttpServletRequestImpl implements HttpServletRequest {
         } else if (asyncStarted) {
             throw UndertowServletMessages.MESSAGES.asyncAlreadyStarted();
         }
-        asyncStarted = true;
         servletRequestContext.setServletRequest(servletRequest);
         servletRequestContext.setServletResponse(servletResponse);
-        return asyncContext = new AsyncContextImpl(exchange, servletRequest, servletResponse, servletRequestContext, true, asyncContext);
+        asyncContext = new AsyncContextImpl(exchange, servletRequest, servletResponse, servletRequestContext, true, asyncContext);
+        asyncStarted = true;
+        return asyncContext;
     }
 
     @Override
